/*-
 * Copyright 2005 Eric Anholt
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *    Eric Anholt <anholt@FreeBSD.org>
 */

#ifndef _PCG_H_
#define _PCG_H_

void pcg_random_seed(int seed);

/* perlin noise functions.  Peristence is < 1, lower is less noise */
float *create_1d_perlin(int size, float persistence, int levels);
void add_1d_perlin(float *data, int size, int offset, int stride, float weight);
float *create_2d_perlin(int size, float persistence, int levels);
void add_2d_perlin(float *data, int size, int ox, int oy, int stride,
    float weight);
void perlin_use_cosine(int use_cosine);

/* Reaction-diffusion systems */
struct turing_reaction {
	int w, h;	/* width/height of arrays */

	/* Chemical concentrations */
	float *a, *b;
	/* beta value -- constant decay of chemical b */
	float *B;
	/* Boolean is-locked value per pixel */
	unsigned char *locks;

	float Da;	/* Rate of diffusion of chemical a */
	float Db;	/* Rate of diffusion of chemical b */
	float s;	/* Reaction rate */

	int curiter;

	/* private members */
	float *da, *db;
	float *dda, *ddb;
};

struct meinhardt_reaction {
	int w, h;	/* width/height of arrays */

	/* Chemical concentrations */
	float *g1, *g2, *r, *s1, *s2;
	/* beta value -- constant decay of chemical b */
	/* Boolean is-locked value per pixel */
	unsigned char *locks;

	float Dg;	/* Rate of diffusion of chemicals g1, g2 */
	float Dr;	/* Rate of diffusion of chemical r */
	float Ds;	/* Rate of diffusion of chemicals s1, s2 */
	float c;
	float gamma;

	int curiter;

	/* private members */
	float A, B;
	float *dg1, *dg2, *dr, *ds1, *ds2;
	float *ddg1, *ddg2, *ddr, *dds1, *dds2;
	float *cs2g1g1, *cs1g2g2, *temp;
};

struct turing_reaction *create_2d_reaction_turing(int w, int h, float Da,
    float Db, float s, float Bbase, float Bvary, float ainit, float binit);
void iterate_2d_reaction_turing(struct turing_reaction *react, int iter);
void free_2d_reaction_turing(struct turing_reaction *react);

struct meinhardt_reaction *create_2d_reaction_meinhardt(int w, int h,
    float c, float Dg, float Ds, float Dr, float gamma,
    float ginit, float sinit, float svary, float rinit);
void iterate_2d_reaction_meinhardt(struct meinhardt_reaction *react, int iter);
void free_2d_reaction_meinhardt(struct meinhardt_reaction *react);

#endif /* _PCG_H_ */
