/*-
 * Copyright 2005 Eric Anholt
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *    Eric Anholt <anholt@FreeBSD.org>
 */

#include <stdio.h>
#include <stdlib.h>
#include "pcg.h"
#include "pcg_int.h"

#if 0
/* Stubs using system random -- should use an OS-independent PRNG here. */

void pcg_random_seed(int seed)
{
	srandom(seed);
}

long pcg_random()
{
	return random();
}

float pcg_random_f()
{
	return ((double)random() / (double)RAND_MAX);
}
#else

/* Park-Miller "minimal standard" 31 bit pseudo-random number generator.
 * See http://www.firstpr.com.au/dsp/rand31
 */
static unsigned int seed = 1;

void pcg_random_seed(int in_seed)
{
	/* Keep people that blindly insert 0 from getting an all-zeroes result.
	 * If they use INT_MAX, oh well.
	 */
	seed = in_seed + 1;
}

long pcg_random()
{
	unsigned int hi, lo;

	lo = 16807 * (seed & 0xffff);
	hi = 16807 * (seed >> 16);

	lo += (hi & 0x7fff) << 16;
	lo += hi >> 15;

	if (lo > 0x7fffffff)
		lo -= 0x7fffffff;

	seed = lo;

	return seed;
}

float pcg_random_f()
{
	float val;

	val = (double)(pcg_random()) / (double)((1UL << 31) - 1);

	/*if (val < 0 || val > 1) {
		printf("bad result %f\n", val);
	}*/

	return val;
}
#endif
