/*-
 * Copyright 2005 Eric Anholt
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *    Eric Anholt <anholt@FreeBSD.org>
 */

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "GL/glut.h"
#include "pcg.h"

static GLuint Window = 0;
static GLuint TexObj[2];
static GLfloat Angle = 0.0f;
static int seed = 0;
static GLboolean color = GL_FALSE;

static void init_tex_seeded(int seed)
{
	pcg_random_seed(seed);

	if (color) {
		GLfloat tex[256][256][3];
		GLint i;

		for (i = 0; i < 3; i++) {
			int x, y;
			float *data = create_2d_perlin(256, .5, 8);
			for (x = 0; x < 256; x++)
				for (y = 0; y < 256; y++)
					tex[x][y][i] = data[x + y * 256];
			free(data);
		}
	
		glTexImage2D(GL_TEXTURE_2D, 0, 3, 256, 256, 0, GL_RGB,
		    GL_FLOAT, tex);
	} else {
		GLfloat *tex;

		tex = create_2d_perlin(256, .5, 8);
		glTexImage2D(GL_TEXTURE_2D, 0, 1, 256, 256, 0, GL_LUMINANCE,
		    GL_FLOAT, tex);
		free(tex);
	}
}

static void draw( void )
{
	glClear( GL_COLOR_BUFFER_BIT );

	glColor3f( 1.0, 1.0, 1.0 );

	/* draw first polygon */
	glPushMatrix();
	glTranslatef( -1.0, 0.0, 0.0 );
	glRotatef( Angle, 0.0, 0.0, 1.0 );
	glBindTexture( GL_TEXTURE_2D, TexObj[0] );
	glBegin( GL_POLYGON );
	glTexCoord2f( 0.0, 0.0 );   glVertex2f( -1.0, -1.0 );
	glTexCoord2f( 1.0, 0.0 );   glVertex2f(  1.0, -1.0 );
	glTexCoord2f( 1.0, 1.0 );   glVertex2f(  1.0,  1.0 );
	glTexCoord2f( 0.0, 1.0 );   glVertex2f( -1.0,  1.0 );
	glEnd();
	glPopMatrix();

	glutSwapBuffers();
}

/* change view Angle, exit upon ESC */
static void key(unsigned char k, int x, int y)
{
	(void) x;
	(void) y;

	switch (k) {
	case 27:
		exit(0);
	case 'c':
		color = !color;
		init_tex_seeded(seed);
		draw();
		break;
	default:
		init_tex_seeded(++seed);
		draw();
	}
}



/* new window size or exposure */
static void reshape( int width, int height )
{
	glViewport(0, 0, (GLint)width, (GLint)height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	/*   glOrtho( -3.0, 3.0, -3.0, 3.0, -10.0, 10.0 );*/
	glFrustum( -2.0, 2.0, -2.0, 2.0, 6.0, 20.0 );
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -8.0 );
}


static void init( void )
{
	glDisable( GL_DITHER );

	/* Setup texturing */
	glEnable( GL_TEXTURE_2D );

	/* generate texture object IDs */
	glGenTextures( 2, TexObj );

	/* setup first texture object */
	glBindTexture( GL_TEXTURE_2D, TexObj[0] );

	init_tex_seeded(seed);

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
}

int main( int argc, char *argv[] )
{
	glutInit(&argc, argv);
	glutInitWindowPosition(0, 0);
	glutInitWindowSize(300, 300);
	glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );

	Window = glutCreateWindow("2d noise texture");
	if (!Window) {
		exit(1);
	}

	init();

	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutDisplayFunc(draw);
	glutMainLoop();

	return 0;
}
