/*-
 * Copyright 2005 Eric Anholt
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Authors:
 *    Eric Anholt <anholt@FreeBSD.org>
 */

#include <assert.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include "pcg.h"

#define TIME_PERLIN 0
#define TIME_MEINHARDT 1

int main(int argc, char **argv)
{
	struct timeval start, end;
	float elapsed;
	int bytes = 0;
	int i;

	gettimeofday(&start, NULL);

#if TIME_PERLIN
#define SIZE 512
	for (i = 0; i < 50; i++) {
		float *data = create_2d_perlin(SIZE, .5, 8);
		bytes += SIZE * SIZE * 4;
		free(data);
		
		free(data);
	}
#elif TIME_MEINHARDT
#define SIZE 256
	do {
		struct meinhardt_reaction *react;
		react = create_2d_reaction_meinhardt(SIZE, SIZE, .002, .016,
		    .08, 0.0, .01, .75, .75, .2, .14);
		iterate_2d_reaction_meinhardt(react, 150);
		free_2d_reaction_meinhardt(react);
		bytes += SIZE * SIZE * 4;
	} while (0);
#endif

	gettimeofday(&end, NULL);

	elapsed = (end.tv_sec - start.tv_sec);
	elapsed += (float)(end.tv_usec - start.tv_usec) / 1000000.0;

	printf("%d bytes calculated in %01.04f secs (%.1f KB/sec)\n", bytes,
	    elapsed, (float)bytes / 1024.0 / elapsed);

	return 0;
}
